unit MyObj;

interface

uses Crt, MyGraph, Objects, General, graph;

const PeakInd : array [0..5,0..3] of byte = ((3,2,1,0),
                                             (3,7,6,2),
                                             (7,4,5,6),
                                             (4,0,1,5),
                                             (1,2,6,5),
                                             (0,4,7,3));

type

     TMyPoint = object(TObject)
       Drawing, Original : TCoordinates;
       Angle : TCoordinates;
       constructor Init(pPoint : TCoordinates);
       procedure Rotate(dAngle : TCoordinates; Center : TCoordinates);
       procedure Reset;
       destructor Done; virtual;
     end;

     TCube = object(TObject)
      public
       Peaks : array [0..7] of TMyPoint;
       Angle : TCoordinates;
       Center : TCoordinates;
       Normals : array [0..5] of TMyPoint;
       Indexes : array [0..5] of array [0..3] of byte;
       {IntoCenter : boolean; }
       constructor Init(LeftTop, RightBot : TCoordinates);        {Canvas?}
       procedure Rotate(dAngle : TCoordinates);
       procedure Reset;
       procedure Draw(Color : byte);
       procedure Hide;
       procedure ShiftToCenter;
       destructor Done; virtual;
     end;

     TMyLine = object(TObject)
       Peaks : array [0..1] of TMyPoint;
       CutPoint : array [0..1] of TMyPoint;
       Angle : TCoordinates;
       t1, t2 : real;
       Internal : boolean;
   {    IntoCenter : boolean;}
       Cutted : boolean;
       constructor Init(Fr, Sn : TCoordinates);
       procedure Rotate(dAngle : TCoordinates;
                        Center : TCoordinates);
       procedure Reset;
       procedure Draw(Color : byte);
       procedure Hide;
       procedure ShiftToCenter;
       procedure CutOff(CCube : TCube);
       destructor Done; virtual;

     end;

implementation

{------------ ᠭ ⮤  TMyPoint ------------}

constructor TMyPoint.Init(pPoint : TCoordinates);
begin
  Original.X := pPoint.X;
  Original.Y := pPoint.Y;
  Original.Z := pPoint.Z;
  Drawing.X := pPoint.X;
  Drawing.Y := pPoint.Y;
  Drawing.Z := pPoint.Z;
  Angle.X := 0;
  Angle.Y := 0;
  Angle.Z := 0;
end;

procedure TMyPoint.Rotate(dAngle : TCoordinates; Center : TCoordinates);
var
     AngX, AngY, AngZ : real;
     temp1, temp2 : TCoordinates;
begin
  Angle.X := (Angle.X + dAngle.X) mod 360;
  Angle.Y := (Angle.Y + dAngle.Y) mod 360;
  Angle.Z := (Angle.Z + dAngle.Z) mod 360;
  AngX := pi/180 * Angle.X;
  AngY := pi/180 * Angle.Y;
  AngZ := pi/180 * Angle.Z;
  {   }
  temp2.X := Original.X - Center.X;
  temp2.Y := Original.Y - Center.Y;
  temp2.Z := Original.Z - Center.Z;
  Drawing.X := temp2.X;
  Drawing.Y := round(temp2.Y * cos(AngX) + temp2.Z * sin(AngX));
  Drawing.Z := round(-temp2.Y * sin(AngX) + temp2.Z * cos(AngX));
  {   Y}
  temp1.X := round(Drawing.X * cos(AngY) - Drawing.Z * sin(AngY));
  temp1.Z := round(Drawing.X * sin(AngY) + Drawing.Z * cos(AngY));
  Drawing.X := temp1.X;
  Drawing.Z := temp1.Z;
  {   Z}
  temp1.X := round(Drawing.X * cos(AngZ) + Drawing.Y * sin(AngZ));
  temp1.Y := round(-Drawing.X * sin(AngZ) + Drawing.Y * cos(AngZ));
  Drawing.X := temp1.X;
  Drawing.Y := temp1.Y;

  Drawing.X := Drawing.X + Center.X;
  Drawing.Y := Drawing.Y + Center.Y;
  Drawing.Z := Drawing.Z + Center.Z;

end;

procedure TMyPoint.Reset;
begin
  Angle.X := 0;
  Angle.Y := 0;
  Angle.Z := 0;
  Drawing.X := Original.X;
  Drawing.Y := Original.Y;
  Drawing.Z := Original.Z;
end;

destructor TMyPoint.Done;
begin
  inherited Done;
end;

{------------ ᠭ ⮤  TMyPoly ------------}
constructor TCube.Init(LeftTop, RightBot : TCoordinates);                   {Canvas?}

var pTemp, A, B : TCoordinates;
    i, j : byte;


begin

 pTemp.X := LeftTop.X;
 pTemp.Y := LeftTop.Y;
 pTemp.Z := LeftTop.Z;

 Peaks[0].Init(pTemp);

 pTemp.Y := RightBot.Y;
 Peaks[1].Init(pTemp);

 pTemp.X :=  RightBot.X;
 Peaks[2].Init(pTemp);

 pTemp.Y := LeftTop.Y;
 Peaks[3].Init(pTemp);

 pTemp.X := LeftTop.X;
 pTemp.Z := RightBot.Z;
 Peaks[4].Init(pTemp);

 pTemp.Y := RightBot.Y;
 Peaks[5].Init(pTemp);

 pTemp.X := RightBot.X;
 Peaks[6].Init(pTemp);

 pTemp.Y := LeftTop.Y;
 Peaks[7].Init(pTemp);

 for i := 0 to 5 do
   for j := 0 to 3 do
     Indexes[i][j] := PeakInd[i,j];

 for i := 0 to 5 do
   begin
     A.X := Peaks[Indexes[i][1]].Original.X - Peaks[Indexes[i][0]].Original.X;
     A.Y := Peaks[Indexes[i][1]].Original.Y - Peaks[Indexes[i][0]].Original.Y;
     A.Z := Peaks[Indexes[i][1]].Original.Z - Peaks[Indexes[i][0]].Original.Z;

     B.X := Peaks[Indexes[i][2]].Original.X - Peaks[Indexes[i][1]].Original.X;
     B.Y := Peaks[Indexes[i][2]].Original.Y - Peaks[Indexes[i][1]].Original.Y;
     B.Z := Peaks[Indexes[i][2]].Original.Z - Peaks[Indexes[i][1]].Original.Z;

     pTemp.X := A.Y*B.Z - A.Z*B.Y;
     pTemp.Y := A.Z*B.X - A.X*B.Z;
     pTemp.Z := A.X*B.Y - A.Y*B.X;

     Normals[i].Init(pTemp);
   end;

  Center.X := (RightBot.X - LeftTop.X) div 2;
  Center.Y := (RightBot.Y - LeftTop.Y) div 2;
  Center.Z := (RightBot.Z - LeftTop.Z) div 2;

  Angle.X := 0;
  Angle.Y := 0;
  Angle.Z := 0;

  {IntoCenter := true;}

  Reset;

end;

procedure TCube.Rotate(dAngle : TCoordinates);

var i : byte;

begin

  Angle.X := (dAngle.X + Angle.X) mod 360;
  Angle.Y := (dAngle.Y + Angle.Y) mod 360;
  Angle.Z := (dAngle.Z + Angle.Z) mod 360;

  for i := 0 to 7 do
    Peaks[i].Rotate(dAngle, Center);

  for i := 0 to 5 do
    Normals[i].Rotate(dAngle, Center);

end;

procedure TCube.Reset;
var
    i : byte;
begin

 for i := 0 to 7 do
   Peaks[i].Reset;

 for i := 0 to 5 do
   Normals[i].Reset;

 Angle.X := 0;
 Angle.Y := 0;
 Angle.Z := 0;

end;

procedure TCube.Draw(Color : byte);
var
    Poly : array [0..5] of array [0..3] of TCoordinates;
    i, j : byte;
begin
  MySetColor(Color);
  for i := 0 to 5 do
    begin
     for j := 0 to 3 do
       Poly[i][j] := Peaks[Indexes[i][j]].Drawing;
     if Normals[i].Drawing.Z > 0 then MyDrawPoly(Poly[i],1);
    end;
  for i := 0 to 5 do
     if Normals[i].Drawing.Z < 0 then MyDrawPoly(Poly[i],0);
end;

procedure TCube.ShiftToCenter;
var
    i : byte;
begin
  for i := 0 to 7 do
    begin
     Peaks[i].Drawing.X := Peaks[i].Drawing.X + 200;
     Peaks[i].Drawing.Y := Peaks[i].Drawing.Y + 320;
    end;
end;

procedure TCube.Hide;
begin
  Draw(0);
end;

destructor TCube.Done;
begin
  inherited Done;
end;



constructor TMyLine.Init(Fr, Sn : TCoordinates);
var
     i : byte;
begin

  Peaks[0].Init(Fr);
  Peaks[1].Init(Sn);

  CutPoint[0].Init(Fr);
  CutPoint[1].Init(Sn);

  Angle.X := 0;
  Angle.Y := 0;
  Angle.Z := 0;

  t1 := 0; t2 := 1;

  internal := true;
  Cutted := false;

  Reset;

end;


procedure TMyLine.Rotate(dAngle : TCoordinates;
             Center : TCoordinates);
var
    i : byte;
begin
    for i := 0 to 1 do
     begin
      Peaks[i].Rotate(dAngle,Center);
      CutPoint[i].Rotate(dAngle,Center);
     end;

end;

procedure TMyLine.Reset;
var
    i : byte;
begin
  for i := 0 to 1 do
    begin
      Peaks[i].Reset;
      CutPoint[i].Reset;
    end;

  Angle.X := 0;
  Angle.Y := 0;
  Angle.Z := 0;

  Internal := true;
  Cutted := false;
  t1 := 0; t2 := 1;

end;

procedure TMyLine.Draw(Color : byte);
var
    DrwLine : array [0..1] of TCoordinates;
    dx, dy : integer;
begin
  MySetColor(Color);
  if not Cutted then
    begin
     DrwLine[0] := Peaks[0].Drawing;
     DrwLine[1] := Peaks[1].Drawing;
     MyLine(DrwLine,0);
    end
   else
    if internal then
      begin
       DrwLine[0] := CutPoint[0].Drawing;
       DrwLine[1] := CutPoint[1].Drawing;
       MyLine(DrwLine,1);
      end
     else
      begin
       DrwLine[0] := Peaks[0].Drawing;
       DrwLine[1] := CutPoint[0].Drawing;
       MyLine(DrwLine,0);
       DrwLine[0] := CutPoint[1].Drawing;
       DrwLine[1] := Peaks[1].Drawing;
       MyLine(DrwLine,0);
      end;
end;

procedure TMyLine.Hide;
begin
  Draw(0);
end;

procedure TMyLine.ShiftToCenter;
var
    i : byte;
begin
  for i := 0 to 1 do
    begin
     Peaks[i].Drawing.X := Peaks[i].Drawing.X + 200;
     Peaks[i].Drawing.Y := Peaks[i].Drawing.Y + 320;
     CutPoint[i].Drawing.X := CutPoint[i].Drawing.X + 200;
     CutPoint[i].Drawing.Y := CutPoint[i].Drawing.Y + 320;
    end;
end;

procedure TMyLine.CutOff(CCube : TCube);
var
    t, tl, th : real;
    ind : byte;
    D, W, Middle : TCoordinates;
    MultiD, MultiW : longint;

begin
  D.X := Peaks[1].Original.X - Peaks[0].Original.X;
  D.Y := Peaks[1].Original.Y - Peaks[0].Original.Y;
  D.Z := Peaks[1].Original.Z - Peaks[0].Original.Z;
{  writeln('First line parametrs: ');
  writeln('D.X: ',D.X,'  D.Y: ',D.Y,'  D.Z: ',D.Z);
  readkey;                                         }
  tl := 0; th := 1;
  for ind := 0 to 5 do
    begin
{      writeln('Processing with ',ind,' poly...');}
      W.X := Peaks[0].Original.X - CCube.Peaks[CCube.Indexes[ind][0]].Original.X;
      W.Y := Peaks[0].Original.Y - CCube.Peaks[CCube.Indexes[ind][0]].Original.Y;
      W.Z := Peaks[0].Original.Z - CCube.Peaks[CCube.Indexes[ind][0]].Original.Z;
 {     writeln('Number of first point: ',CCube.Indexes[ind][0]);
      writeln('W.X: ',W.X,'  W.Y: ',W.Y,'  W.Z: ',W.Z);         }
      MultiD := ScalarMult(D,CCube.Normals[ind].Original);
      MultiW := ScalarMult(W,CCube.Normals[ind].Original);
{     writeln('Multiplication D*Normals: ',MultiD);
      writeln('Multiplication W*Normals: ',MultiW);}
      if MultiD<>0 then
        begin
         t := -(MultiW/MultiD);
        { writeln(t:3:4);}
         if MultiD > 0 then
            if (t < 1) then tl := max(tl,t) else begin end
           else
            if (t > 0) then th := min(th,t) else begin end;
        end
       else
        begin
        end;
    end;

  Cutted := true;

  if tl>th then
      Cutted := false
   else
    begin
      t := (tl+th)/2;
      Middle.X := round(D.X*t) + Peaks[0].Original.X;
      Middle.Y := round(D.Y*t) + Peaks[0].Original.Y;
      Middle.Z := round(D.Z*t) + Peaks[0].Original.Z;
      for ind := 0 to 5 do
        begin
         W.X := Middle.X - CCube.Peaks[CCube.Indexes[ind][0]].Original.X;
         W.Y := Middle.Y - CCube.Peaks[CCube.Indexes[ind][0]].Original.Y;
         W.Z := Middle.Z - CCube.Peaks[CCube.Indexes[ind][0]].Original.Z;
         MultiW := ScalarMult(W,CCube.Normals[ind].Original);
         if MultiW < 0 then
           begin
             Cutted := false;
             break;
           end;
        end;
    end;

  if Cutted then
    begin
      t1 := tl;
      t2 := th;
    end
   else
    begin
     t1 := 0;
     t2 := 1;
    end;

  CutPoint[0].Original.X := round(D.X*t1) + Peaks[0].Original.X;
  CutPoint[0].Original.Y := round(D.Y*t1) + Peaks[0].Original.Y;
  CutPoint[0].Original.Z := round(D.Z*t1) + Peaks[0].Original.Z;

  CutPoint[1].Original.X := round(D.X*t2) + Peaks[0].Original.X;
  CutPoint[1].Original.Y := round(D.Y*t2) + Peaks[0].Original.Y;
  CutPoint[1].Original.Z := round(D.Z*t2) + Peaks[0].Original.Z;

  CutPoint[0].Reset;
  CutPoint[1].Reset;

end;

destructor TMyLine.Done;
begin
  inherited Done;
end;

End.